@@ -2,20 +2,21 @@
 #'
 #' This function registers a new geometry of territorial units into the
 #' geospatial database.
-#' @param nation [\code{character(1)}]\cr either the nation name or the column
-#'   of the file's attribute table that contains nations.
+#' @param ... [\code{character(1)}]\cr optional named argument selecting the
+#'   main territory into which this geometry is nested. The name of this must be
+#'   a class of the gazetteer and the value must be one of the territory names
+#'   of that class, e.g. \emph{nation = "Estonia"}.
 #' @param subset [\code{character(1)}]\cr optional argument to specify which
 #'   subset the file contains. This could be a subset of territorial units (e.g.
 #'   only one municipality) or of a target variable.
 #' @param gSeries [\code{character(1)}]\cr the name of the geometry dataseries
 #'   (see \code{\link{regDataseries}}).
-#' @param level [\code{integerish(1)}]\cr the administrative level at which the
-#'   geometry is recorded.
+#' @param label [\code{list(.)}]\cr list of as many columns as there are in
+#'   common in the ontology and this geometry. Must be of the form
+#'   \code{list(class = columnName)}, with 'class' as the class of the ontology
+#'   corresponding to the respective column name in the geometry.
 #' @param layer [\code{character}]\cr the name of the file's layer from which
 #'   the geometry should be created (if applicable).
-#' @param nameCol [\code{character(.)}]\cr the columns in which the names of
-#'   administrative units are to be found, delimited by \code{"|"}; see
-#'   Examples.
 #' @param archive [\code{character(1)}]\cr the original file (perhaps a *.zip)
 #'   from which the geometry emerges.
 #' @param archiveLink [\code{character(1)}]\cr download-link of the archive.
@@ -34,46 +35,46 @@
 #' @param overwrite [\code{logical(1)}]\cr whether or not the geometry to
 #'   register shall overwrite a potentially already existing older version.
 #' @details When processing geometries to which areal data shall be linked,
-#'   carry out the following steps: \enumerate{ \item Determine the
-#'   \code{nation}, a \code{subset} (if applicable), the dataseries of the
-#'   geometry and the administrative \code{level}, and provide them as arguments
-#'   to this function. \item Run the function. \item Export the shapefile with
-#'   the following properties: \itemize{ \item Format: GeoPackage \item File
-#'   name: What is provided as message by this function \item CRS: EPSG:4326 -
-#'   WGS 84 \item make sure that 'all fields are exported'} \item Confirm that
-#'   you have saved the file.}
+#'   carry out the following steps: \enumerate{ \item Determine the main
+#'   territory (such as a nation, or any other polygon), a \code{subset} (if
+#'   applicable), the dataseries of the geometry and the ontology \code{label},
+#'   and provide them as arguments to this function. \item Run the function.
+#'   \item Export the shapefile with the following properties: \itemize{ \item
+#'   Format: GeoPackage \item File name: What is provided as message by this
+#'   function \item CRS: EPSG:4326 - WGS 84 \item make sure that 'all fields are
+#'   exported'} \item Confirm that you have saved the file.}
 #' @return Returns a tibble of the entry that is appended to
 #'   'inv_geometries.csv' in case \code{update = TRUE}.
+#' @family register functions
 #' @examples
-#' # build the example database
-#' makeExampleDB(until = "regDataseries")
+#' if(dev.interactive()){
+#'   # build the example database
+#'   makeExampleDB(until = "regDataseries", path = tempdir())
 #'
-#' # The GADM dataset comes as *.zip archive
-#' regGeometry(nation = "NAME_0",
-#'             gSeries = "gadm",
-#'             level = 1,
-#'             layer = "example_geom1",
-#'             nameCol = "NAME_0",
-#'             archive = "example_geom.7z|example_geom1.gpkg",
-#'             archiveLink = "https://gadm.org/",
-#'             nextUpdate = "2019-10-01",
-#'             updateFrequency = "quarterly",
-#'             update = TRUE)
+#'   # The GADM dataset comes as *.7z archive
+#'   regGeometry(gSeries = "gadm",
+#'               label = list(al1 = "NAME_0"),
+#'               layer = "example_geom1",
+#'               archive = "example_geom.7z|example_geom1.gpkg",
+#'               archiveLink = "https://gadm.org/",
+#'               nextUpdate = "2019-10-01",
+#'               updateFrequency = "quarterly",
+#'               update = TRUE)
 #'
-#' # The second administrative level in GADM contains names in the columns
-#' # NAME_0 and NAME_1
-#' regGeometry(nation = "NAME_0",
-#'             gSeries = "gadm",
-#'             level = 2,
-#'             layer = "example_geom2",
-#'             nameCol = "NAME_0|NAME_1",
-#'             archive = "example_geom.7z|example_geom2.gpkg",
-#'             archiveLink = "https://gadm.org/",
-#'             nextUpdate = "2019-10-01",
-#'             updateFrequency = "quarterly",
-#'             update = TRUE)
+#'   # The second administrative level in GADM contains names in the columns
+#'   # NAME_0 and NAME_1
+#'   regGeometry(gSeries = "gadm",
+#'               label = list(al1 = "NAME_0", al2 = "NAME_1"),
+#'               layer = "example_geom2",
+#'               archive = "example_geom.7z|example_geom2.gpkg",
+#'               archiveLink = "https://gadm.org/",
+#'               nextUpdate = "2019-10-01",
+#'               updateFrequency = "quarterly",
+#'               update = TRUE)
+#' }
 #' @importFrom checkmate assertNames assertCharacter assertIntegerish
-#'   assertFileExists testChoice assertLogical
+#'   assertFileExists testChoice assertLogical testSubset
+#' @importFrom ontologics load_ontology
 #' @importFrom readr read_csv
 #' @importFrom dplyr filter distinct
 #' @importFrom stringr str_split
@@ -81,19 +82,21 @@
 #' @importFrom tibble tibble
 #' @export
 
-regGeometry <- function(nation = NULL, subset = NULL, gSeries = NULL, level = NULL,
-                        layer = NULL, nameCol = NULL, archive = NULL, archiveLink = NULL,
+regGeometry <- function(..., subset = NULL, gSeries = NULL, label = NULL,
+                        layer = NULL, archive = NULL, archiveLink = NULL,
                         nextUpdate = NULL, updateFrequency = NULL, notes = NULL,
                         update = FALSE, overwrite = FALSE){
 
   # set internal paths
   intPaths <- paste0(getOption(x = "adb_path"))
+  gazPath <- paste0(getOption(x = "gazetteer_path"))
+  topClass <- paste0(getOption(x = "gazetteer_top"))
 
   # get tables
   inv_dataseries <- read_csv(paste0(intPaths, "/inv_dataseries.csv"), col_types = "icccccc")
-  inv_geometries <- read_csv(paste0(intPaths, "/inv_geometries.csv"), col_types = "iiiccccccDccc")
+  inv_geometries <- read_csv(paste0(intPaths, "/inv_geometries.csv"), col_types = "iicccccDDcc")
 
-   if(dim(inv_dataseries)[1] == 0){
+  if(dim(inv_dataseries)[1] == 0){
     stop("'inv_dataseries.csv' does not contain any entries!")
   }
 
@@ -101,53 +104,46 @@ regGeometry <- function(nation = NULL, subset = NULL, gSeries = NULL, level = NU
   testing <- getOption(x = "adb_testing")
 
   # check validity of arguments
-  assertNames(x = colnames(inv_geometries),
-              permutation.of = c("geoID", "datID", "level", "source_file", "layer",
-                                 "nation_column", "unit_column", "orig_file", "orig_link", "download_date",
-                                 "next_update", "update_frequency", "notes"))
-  assertCharacter(x = nation, ignore.case = TRUE, any.missing = FALSE, len = 1, null.ok = TRUE)
+  assertList(x = label, any.missing = FALSE)
   assertCharacter(x = subset, any.missing = FALSE, null.ok = TRUE)
   assertCharacter(x = gSeries, ignore.case = TRUE, any.missing = FALSE, len = 1, null.ok = TRUE)
-  assertIntegerish(x = level, any.missing = FALSE, len = 1, lower = 1, null.ok = TRUE)
   assertCharacter(x = layer, any.missing = FALSE, null.ok = TRUE)
   assertCharacter(x = archive, any.missing = FALSE, null.ok = TRUE)
   assertCharacter(x = archiveLink, any.missing = FALSE, null.ok = TRUE)
-  # maybe assertDate
   assertCharacter(x = nextUpdate, any.missing = FALSE, null.ok = TRUE)
   assertCharacter(x = updateFrequency, any.missing = FALSE, null.ok = TRUE)
   assertCharacter(x = notes, ignore.case = TRUE, any.missing = FALSE, len = 1, null.ok = TRUE)
   assertLogical(x = update, len = 1)
   assertLogical(x = overwrite, len = 1)
+  assertNames(x = colnames(inv_dataseries),
+              permutation.of = c("datID", "name", "description", "homepage",
+                                 "licence_link", "licence_path", "notes"))
+  assertNames(x = colnames(inv_geometries),
+              permutation.of = c("geoID", "datID", "source_file", "layer",
+                                 "label", "orig_file", "orig_link", "download_date",
+                                 "next_update", "update_frequency", "notes"))
 
-  # ask for missing and required arguments
-  if(is.null(nation)){
-    message("please type in either a nation or the name of the column that contains nation names: ")
-    if(!testing){
-      nation <- readline()
-    } else {
-      nation <- countries$nation[11]
-    }
-    if(is.na(nation)){
-      nation = NA_character_
-    }
+  broadest <- exprs(..., .named = TRUE)
+  if(length(broadest) > 0){
+    mainPoly <- broadest[[1]]
+  } else {
+    mainPoly <- ""
   }
 
+  gaz <- load_ontology(gazPath)
+  gazClasses <- get_class(ontology = gazPath)
+  # include here a test that checks whether broaders is actually part of the gazetteer
+  # if(!testSubset(x = names(broadest), choices = unique(gaz$attributes$class))){
+  #   stop("please specify a main category that is part of the classes in the gazetteer (",
+  #        paste0(unique(gaz$attributes$class)[1:4], collapse = ", "), ")")
+  # }
+
   if(!is.null(subset)){
     if(grepl(pattern = "_", x = subset)){
       stop("please give a subset that does not contain any '_' characters.")
     }
   }
 
-  if(is.null(nameCol)){
-    message("please type in the name of the column that contains unit names: ")
-    if(!testing){
-      nameCol <- readline()
-    } else {
-      nameCol <- "units"
-    }
-  }
-
-
   if(is.null(gSeries)){
     message("please type in to which series the geometry belongs: ")
     if(!testing){
@@ -174,17 +170,21 @@ regGeometry <- function(nation = NULL, subset = NULL, gSeries = NULL, level = NU
     }
   }
 
-  if(is.null(level)){
-    message("please type in the administrative level of the units: ")
-    if(!testing){
-      level <- readline()
-    } else {
-      level <- 1
-    }
-    if(is.na(level)){
-      level = NA_integer_
-    }
-  }
+  # if(is.null(label)){
+  #   message("please type in the ontology label and the column name of the units (e.g. al1=NAME_0: ")
+  #   if(!testing){
+  #     theLabel <- readline()
+  #   } else {
+  #     theLabel <- "al1"
+  #   }
+  #   if(is.na(theLabel)){
+  #     theLabel = NA_character_
+  #   }
+  # } else {
+    assertSubset(x = names(label), choices = gazClasses$label)
+    theLabel <- tail(names(label), 1)
+    labelString <- paste0(paste0(names(label), "=", label), collapse = "|")
+  # }
 
   if(is.null(archive)){
     message("please type in the archives' file name: ")
@@ -198,22 +198,8 @@ regGeometry <- function(nation = NULL, subset = NULL, gSeries = NULL, level = NU
     }
   }
 
-  # determine nation value
-  if(!testChoice(x = nation, choices = countries$nation)){
-    theNation <- NULL
-  } else{
-    nations <- nation
-    assertChoice(x = nations, choices = countries$nation)
-    theNation <- countries %>%
-      as_tibble() %>%
-      filter(nation == nations) %>%
-      distinct(iso_a3) %>%
-      tolower()
-    nation <- ""
-  }
-
   # put together file name and get confirmation that file should exist now
-  fileName <- paste0(theNation, "_", level, "_", subset, "_", gSeries, ".gpkg")
+  fileName <- paste0(mainPoly, "_", theLabel, "_", subset, "_", gSeries, ".gpkg")
   filePath <- paste0(intPaths, "/adb_geometries/stage2/", fileName)
   filesTrace <- str_split(archive, "\\|")[[1]]
 
@@ -313,16 +299,11 @@ regGeometry <- function(nation = NULL, subset = NULL, gSeries = NULL, level = NU
 
     # to check that what has been given in 'nation' and 'nameCol' is in fact a
     # column in the geometry, load it
-    if(is.null(theNation)){
-      theGeometry <- read_sf(dsn = filePath,
-                             stringsAsFactors = FALSE)
-      assertChoice(x = nation, choices = colnames(theGeometry))
+    if(is.null(mainPoly)){
+      theGeometry <- read_sf(dsn = filePath, stringsAsFactors = FALSE)
     }
 
-    nameCols <- str_split(string = nameCol, pattern = "\\|")[[1]]
-    if(length(nameCols) != level){
-      warning("'nameCol' contains less entries (", length(nameCols), ") than implied by the level (", level, ")")
-    }
+    nameCols <- unlist(label, use.names = FALSE)
 
     # determine which layers exist and ask the user which to chose, if none is
     # given
@@ -344,11 +325,9 @@ regGeometry <- function(nation = NULL, subset = NULL, gSeries = NULL, level = NU
     # construct new documentation
     doc <- tibble(geoID = newGID,
                   datID = dataSeries,
-                  level = level,
                   source_file = fileName,
                   layer = layer,
-                  nation_column = nation,
-                  unit_column = nameCol,
+                  label = labelString,
                   orig_file = archive,
                   orig_link = archiveLink,
                   download_date = Sys.Date(),
@@ -358,10 +337,10 @@ regGeometry <- function(nation = NULL, subset = NULL, gSeries = NULL, level = NU
     if(!any(inv_geometries$source_file %in% fileName) | overwrite){
       # in case the user wants to update, attach the new information to the table
       # inv_geometries.csv
-      updateTable(index = doc, name = "inv_geometries", matchCols = c("source_file", "level"))
+      updateTable(index = doc, name = "inv_geometries", matchCols = c("source_file", "label"))
     }
     return(doc)
   } else {
     message(paste0("... the filename is '", fileName, "'."))
   }
-}
+}
\ No newline at end of file
